import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';

// import data for GitHub Gist
const csvUrl =
  'https://gist.githubusercontent.com/bhargavarathod/4e47c95f3ac3e8916c40d105ea530847/raw/ed63ec8f812cb61ed22bc3f663724661325fb0e1/iris.csv';

// storing data in each variable
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

// define width and height
const width = window.innerWidth;
const height = window.innerHeight;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const xValue = (d) => d.petal_length;
const yValue = (d) => d.petal_width;
const rxValue = (d) => d.sepal_length;
const ryValue = (d) => d.sepal_width;
const group = (d) => d.species;

// create plot 
const main = async () => {
  const data = await csv(csvUrl, parseRow);

  const x = xValue((d) => d.petal_width);

  const y = yValue((d) => d.sepal_length);

  const color = d3
    .scaleOrdinal()
    .domain(['setosa', 'versicolor', 'virginica'])
    .range(['red ', 'green', 'blue']);
  
  // adding colors to the scatter plot

  const marks = data.map((d) => ({
    x: xValue((d) => d.petal_width),
    y: yValue((d) => d.sepal_length),
    color: color(group(d)),
    rx: rxValue(d),
    ry: ryValue(d),
  }));
  
  // plotting the scatter plot
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.petal_width)
    .yValue((d) => d.sepal_length)
    .margin({
      top: 50,
      right: 50,
      bottom: 90,
      left: 90,
    })
    .radius(4);   // defin radius of circle in scatterplot
  
  // define Title
  svg
    .append('text')
    .attr('x', 450)
    .attr('y', 20)
    .text('Reusable Iris Scatterplot')
    .style('font-size', '23px')
  	.style("font-weight","bold")
    .attr('alignment-baseline', 'middle');
  
  
  //drawing circles and styling
  svg
    .selectAll('circle')
    .data(marks)
    .join('circle')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('fill', (d) => d.color)
    .attr('rx', (d) => d.rx)
    .attr('ry', (d) => d.ry);
  
  // adding the Y label
  svg
    .append('text')
    .attr('x', -300)
    .attr('y', 20)
    .attr('font-size', '15px')
    .attr('transform', 'rotate(-90)')
  .style("font-weight","bold")
    .text('Sepal Length');

  // adding the X label
  svg
    .append('text')
    .attr('x', 500)
    .attr('y', 475)
    .attr('font-size', '15px')
  .style("font-weight","bold")
    .text('Petal Length');

  const columns = [
    'petal_width',
    'sepal_width',
    'petal_length',
    'sepal_length',
  ];
  let i = 0;
  setInterval(() => {
    plot.xValue(
      (d) => d[columns[i % columns.length]]
    );
    svg.call(plot);
    i++;
  }, 2000); // every 2 seconds change the visualization
};
main();
